CREATE OR REPLACE PACKAGE MCL_COMPRESS_CALC AS
/*
* Package:    MCL_COMPRESS_CALC
* Date:       29-04-2011
* Author:     M.Lambrechts <marcel.lambrechts@gmail.com>
* Version:    1.1
* 
* Notes
* =====
* Important: The index compression calculator uses ANALYZE INDEX ... VALIDATE STRUCTURE which can have big implications
*            on the availability of the index you want to be advised on (due to locking) !!
*
* Required privileges:
*      - GRANT SELECT ON SYS.V_$INSTANCE TO ...
*      - GRANT SELECT ON SYS.V_$PARAMETER TO ...
*      - GRANT SELECT ON SYS.DBA_SEGMENTS TO ...
*      - GRANT SELECT ON SYS.DBA_TABLESPACES TO ...
*      - GRANT SELECT ON <table you want to run the advisor on> TO ... or GRANT SELECT ANY TABLE TO ...
*      - GRANT ANALYZE ANY TO ...
*      - GRANT CREATE TABLE TO ...
*
* Required quota on tablespaces:
*      - You should have the quota for the used tablespace set high enough to store the sample tables
*        that are created when you run the calculator for a table or table partition
*
* Description
* ===========
* This package can be used to determine the compression ratio for tables, table partitions or indexes. You can use
* it "online" or by using a worklist where you add the objects you want to be advised on in a worklist table
* and read the results for these objects in a stats table.
*
* Changes:
* Date			Who		Version		What
* =============================================================================
* 29-04-2011	ML		1.0			Initial creation
* 10-05-2011	ML		1.0			Released
* 11-05-2011	ML		1.1			Added ADD_TO_WORKLIST procedure definition
* 11-05-2011	ML		1.1			Changed argument names for procedure
*									STORE_COMPRESS_STATS
* =============================================================================
*
*/

/*
*    Constant declaration
*/
AVG_COMP_FACTOR CONSTANT NUMBER := 2.5;
DEF_SAMP_BLOCKS CONSTANT NUMBER := 1000;
SIZE_MB         CONSTANT NUMBER := 1048576;
SIZE_GB         CONSTANT NUMBER := 1073741824;
WORKLIST_TABLE	CONSTANT VARCHAR2(30) := 'MCL_COMP_CALC_WORKLIST';
STATS_TABLE		CONSTANT VARCHAR2(30) := 'MCL_COMP_CALC_STATS';

PROCEDURE GET_RATIO_FOR_TABLE
  (owner IN VARCHAR2, tablename IN VARCHAR2, partname IN VARCHAR2 DEFAULT NULL,
   compress_type IN VARCHAR2 DEFAULT 'DIRECT_LOAD', sample_pct IN NUMBER DEFAULT NULL,
   samp_tsname IN VARCHAR2 DEFAULT NULL, show_samp_usage IN BOOLEAN DEFAULT FALSE,
   store_stats IN BOOLEAN DEFAULT FALSE);
/*
* Procedure: GET_RATIO_FOR_TABLE
*
* This procedure will show the compression ratio for a table or a table partition.
*
* Mandatory arguments:
*  owner				Owner of the object to get the compression ratio for
*  tablename			Name of the table or table partition to get compression ratio for
*
* Optional arguments:
*  partname				Name of the table partition you want to get the compression ratio for
*  compress_type		DIRECT_LOAD or OLTP - DIRECT_LOAD is the default
*  sample_pct			Sample size (%) of the source table to use - value between 0.000001 and 100)
*  samp_tsname			Which tablespace should be used for temporary storing the sample tables
*  show_samp_usage		Show the storage that would be used for the sample tables. When set to TRUE
*						this procedure won't actually calculate the compression ratio
*  store_stats			Store the results in the stats table (mcl_comp_calc_stats)
*/

PROCEDURE GET_RATIO_FOR_INDEX
  (owner IN VARCHAR2, indexname IN VARCHAR2, store_stats IN BOOLEAN DEFAULT FALSE);
/*
* Procedure: GET_RATIO_FOR_INDEX
*
* This procedure will show the compression ratio for the specified index.
* Mandatory arguments:
* owner					Owner of the index to get the compression ratio and prefix length for
* indexname				Name of the index to get the compression ratio and prefix length for
*
* Optional arguments:
*  store_stats			Store the results in the stats table (mcl_comp_calc_stats)
*/

PROCEDURE PROCESS_WORKLIST;
/*
* Procedure: PROCESS_WORKLIST
*
* This procedure will read information about tables/table partitions or indexes from the worklist table
* and will try to calculate the compression ratio for each of these objects and the prefix length
* for indexes. The procedure will store the results for each object in the stats table (mcl_comp_calc_stats).
* It will update the status of each object in the worklist table, so you can follow the progress of its work.
*/

PROCEDURE ADD_TO_WORKLIST
  (obj_type IN VARCHAR2, obj_owner IN VARCHAR2, obj_name IN VARCHAR2, obj_partition IN VARCHAR2 DEFAULT NULL, compress_type IN VARCHAR2 DEFAULT NULL,
   sample_pct IN NUMBER DEFAULT NULL, sample_ts IN VARCHAR2 DEFAULT NULL);
/*
* Procedure: ADD_TO_WORKLIST
*
* This procedure will add the specified object to the worklist table generating a unique id for the row.
* Mandatory arguments:
*  obj_type				The type of object (allowed: TABLE, PARTITION or INDEX)
*  obj_owner			The owner of the object to process
*  obj_name				The name of the object to process (table name or index name)
*
* Optional arguments:
*  obj_partition		The name of the partition to process (if obj_type equals PARTITION)
*  compress_type		The type of compression to use (for tables DIRECT_LOAD or OLTP)
*  sample_pct			The sample size percentage to use
*  sample_ts			The name of the tablespace to store the sample tables (table and table partitions)
*/

PROCEDURE STORE_COMPRESS_STATS
  (obj_type IN VARCHAR2, obj_owner IN VARCHAR2, obj_name IN VARCHAR2, obj_partition IN VARCHAR2 DEFAULT NULL, compress_type IN VARCHAR2,
   sample_pct NUMBER DEFAULT 0, prefix_length NUMBER DEFAULT 0, comp_ratio IN NUMBER, org_size IN NUMBER, comp_size IN NUMBER);
/*
* Procedure: STORE_COMPRESS_STATS
*
* This procedure will store the gathered compression statistics into the statistics table.
* Mandatory arguments:
*  obj_type				The type of object (allowed: TABLE, PARTITION or INDEX)
*  obj_owner			The owner of the processed object
*  obj_name				The name of the processed object
*  compress_type		The type of compression (allowed: INDEX KEY, DIRECT_LOAD or OLTP)
*  comp_ratio			The calculated compression ratio
*  org_size				The original size (in bytes) of the processed object
*  comp_size			The size (in bytes) of the object after it would be compressed
*
* Optional arguments:
*  obj_partition		The name of the processed table partition
*  sample_pct			The sample percentage used for processing (only applicable for Tables or Table partitions)
*  prefix_length		The optimal prefix key length when compressing (only applicable for Indexes)
*  
*/

PROCEDURE SETUP_TABLES;
/*
* Procedure: SETUP_TABLES
*
* This procedure will create a worklist table for batch-mode advisory and a statistics
* table for storing statistics about checked tables.
*/

END MCL_COMPRESS_CALC;
/